/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.command;

import java.time.ZonedDateTime;
import java.util.Objects;

import de.ozgcloud.antragsraum.proxy.AntragraumproxyGrpcCommand;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class CommandMapper {
	public static final int IN_MILLISECONDS = 1000;

	static Command fromRestCommand(final AntragraumproxyGrpcCommand command, String nachrichtEventId) {
		var createdAt = ZonedDateTime.now().toEpochSecond() + IN_MILLISECONDS;
		if (Objects.nonNull(command.getCreatedAt())) {
			createdAt = ZonedDateTime.parse(command.getCreatedAt()).toEpochSecond() * IN_MILLISECONDS;
		}

		var finishedAt = Objects.isNull(command.getFinishedAt()) || command.getFinishedAt().isEmpty()
		  ? null
		  : ZonedDateTime.parse(command.getFinishedAt()).toEpochSecond() * IN_MILLISECONDS;

		return Command.builder()
		  .id(command.getId())
		  .nachrichtEventId(nachrichtEventId)
		  .status(command.getStatus())
		  .createdAt(createdAt)
		  .finishedAt(finishedAt)
		  .build();
	}
}
