/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import java.io.IOException;
import java.util.UUID;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.saml2.provider.service.authentication.Saml2Authentication;
import org.springframework.security.web.DefaultRedirectStrategy;

import lombok.extern.log4j.Log4j2;

@Log4j2
class SamlRedirectStrategy extends DefaultRedirectStrategy {
	private final UserDetailsService userDetailsService;
	private final String redirectUrl;
	private String onetimeToken;

	public SamlRedirectStrategy(UserDetailsService userDetailsService, String redirectUrl) {
		super();
		this.userDetailsService = userDetailsService;
		this.redirectUrl = redirectUrl;
	}

	public void sendRedirect(HttpServletRequest request, HttpServletResponse response, String url, Authentication authentication) throws IOException {
		onetimeToken = UUID.randomUUID().toString();
		if (authentication instanceof Saml2Authentication saml2Authentication) {
			((InMemoryUserDetailService) userDetailsService).addUser(onetimeToken, UserMapper.map(saml2Authentication));
		} else {
			LOG.error("Invalid authentication received. Is {} expected {}", authentication.getClass().getName(), Saml2Authentication.class.getName());
		}

		sendRedirect(request, response, url);
	}

	@Override
	public void sendRedirect(HttpServletRequest request, HttpServletResponse response, String url) throws IOException {
		String calculateRedirectUrl = null;
		if (StringUtils.isEmpty(redirectUrl)) {
			calculateRedirectUrl = this.calculateRedirectUrl(request.getContextPath(), url) + "?code=" + onetimeToken;
		} else {
			calculateRedirectUrl = redirectUrl + "?code=" + onetimeToken;
		}

		calculateRedirectUrl = response.encodeRedirectURL(calculateRedirectUrl);

		LOG.debug("Redirecting to {}", calculateRedirectUrl);

		response.sendRedirect(calculateRedirectUrl);
	}
}
